# Generated by Django 2.0 on 2018-01-23 17:33

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Blocks',
            fields=[
                ('id', models.CharField(max_length=64, primary_key=True, serialize=False)),
                ('rowid', models.AutoField(db_column='rowId', primary_key=True)),
                ('version', models.IntegerField()),
                ('timestamp', models.IntegerField()),
                ('height', models.IntegerField(unique=True)),
                ('numberoftransactions', models.IntegerField(db_column='numberOfTransactions')),
                ('totalamount', models.BigIntegerField(db_column='totalAmount')),
                ('totalfee', models.BigIntegerField(db_column='totalFee')),
                ('reward', models.BigIntegerField()),
                ('payloadlength', models.IntegerField(db_column='payloadLength')),
                ('payloadhash', models.BinaryField(db_column='payloadHash')),
                ('generatorpublickey', models.BinaryField(db_column='generatorPublicKey')),
                ('blocksignature', models.BinaryField(db_column='blockSignature')),
                ('rawtxs', models.TextField()),
            ],
            options={
                'db_table': 'blocks',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='ForksStat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('delegatepublickey', models.BinaryField(db_column='delegatePublicKey')),
                ('blocktimestamp', models.IntegerField(db_column='blockTimestamp')),
                ('blockid', models.CharField(db_column='blockId', max_length=64)),
                ('blockheight', models.IntegerField(db_column='blockHeight')),
                ('previousblock', models.CharField(db_column='previousBlock', max_length=64)),
                ('cause', models.IntegerField()),
            ],
            options={
                'db_table': 'forks_stat',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MemAccounts',
            fields=[
                ('username', models.CharField(blank=True, max_length=20, null=True)),
                ('isdelegate', models.SmallIntegerField(blank=True, db_column='isDelegate', null=True)),
                ('u_isdelegate', models.SmallIntegerField(blank=True, db_column='u_isDelegate', null=True)),
                ('secondsignature', models.SmallIntegerField(blank=True, db_column='secondSignature', null=True)),
                ('u_secondsignature', models.SmallIntegerField(blank=True, db_column='u_secondSignature', null=True)),
                ('u_username', models.CharField(blank=True, max_length=20, null=True)),
                ('address', models.CharField(max_length=36, primary_key=True, serialize=False)),
                ('publickey', models.BinaryField(blank=True, db_column='publicKey', null=True)),
                ('secondpublickey', models.BinaryField(blank=True, db_column='secondPublicKey', null=True)),
                ('balance', models.BigIntegerField(blank=True, null=True)),
                ('u_balance', models.BigIntegerField(blank=True, null=True)),
                ('vote', models.BigIntegerField(blank=True, null=True)),
                ('rate', models.BigIntegerField(blank=True, null=True)),
                ('delegates', models.TextField(blank=True, null=True)),
                ('u_delegates', models.TextField(blank=True, null=True)),
                ('multisignatures', models.TextField(blank=True, null=True)),
                ('u_multisignatures', models.TextField(blank=True, null=True)),
                ('multimin', models.SmallIntegerField(blank=True, null=True)),
                ('u_multimin', models.SmallIntegerField(blank=True, null=True)),
                ('multilifetime', models.SmallIntegerField(blank=True, null=True)),
                ('u_multilifetime', models.SmallIntegerField(blank=True, null=True)),
                ('blockid', models.CharField(blank=True, db_column='blockId', max_length=64, null=True)),
                ('nameexist', models.SmallIntegerField(blank=True, null=True)),
                ('u_nameexist', models.SmallIntegerField(blank=True, null=True)),
                ('producedblocks', models.IntegerField(blank=True, null=True)),
                ('missedblocks', models.IntegerField(blank=True, null=True)),
                ('fees', models.BigIntegerField(blank=True, null=True)),
                ('rewards', models.BigIntegerField(blank=True, null=True)),
                ('virgin', models.SmallIntegerField(blank=True, null=True)),
            ],
            options={
                'db_table': 'mem_accounts',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MemAccounts2UDelegates',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dependentid', models.CharField(db_column='dependentId', max_length=66)),
            ],
            options={
                'db_table': 'mem_accounts2u_delegates',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MemAccounts2UMultisignatures',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dependentid', models.CharField(db_column='dependentId', max_length=66)),
            ],
            options={
                'db_table': 'mem_accounts2u_multisignatures',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MemDelegates',
            fields=[
                ('publickey', models.CharField(db_column='publicKey', max_length=66, primary_key=True, serialize=False)),
                ('vote', models.BigIntegerField()),
                ('round', models.BigIntegerField()),
                ('producedblocks', models.IntegerField(blank=True, null=True)),
                ('missedblocks', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'db_table': 'mem_delegates',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Migrations',
            fields=[
                ('id', models.CharField(max_length=22, primary_key=True, serialize=False)),
                ('name', models.TextField()),
            ],
            options={
                'db_table': 'migrations',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Peers',
            fields=[
                ('ip', models.GenericIPAddressField(primary_key=True, serialize=False)),
                ('port', models.SmallIntegerField()),
                ('state', models.SmallIntegerField()),
                ('os', models.CharField(blank=True, max_length=64, null=True)),
                ('version', models.CharField(blank=True, max_length=11, null=True)),
                ('clock', models.BigIntegerField(blank=True, null=True)),
            ],
            options={
                'db_table': 'peers',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Transactions',
            fields=[
                ('id', models.CharField(max_length=64, primary_key=True, serialize=False)),
                ('rowid', models.AutoField(db_column='rowId', primary_key=True)),
                ('type', models.SmallIntegerField()),
                ('timestamp', models.IntegerField()),
                ('senderpublickey', models.BinaryField(db_column='senderPublicKey')),
                ('senderid', models.CharField(db_column='senderId', max_length=36)),
                ('recipientid', models.CharField(blank=True, db_column='recipientId', max_length=36, null=True)),
                ('amount', models.BigIntegerField()),
                ('fee', models.BigIntegerField()),
                ('signature', models.BinaryField()),
                ('signsignature', models.BinaryField(blank=True, db_column='signSignature', null=True)),
                ('requesterpublickey', models.BinaryField(blank=True, db_column='requesterPublicKey', null=True)),
                ('vendorfield', models.CharField(blank=True, db_column='vendorField', max_length=64, null=True)),
                ('signatures', models.TextField(blank=True, null=True)),
                ('rawasset', models.TextField(blank=True, null=True)),
            ],
            options={
                'db_table': 'transactions',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tx', models.TextField(max_length=10000)),
                ('desc', models.CharField(max_length=1000)),
                ('confirmations', models.IntegerField(default=0)),
                ('success', models.BooleanField(default=False)),
                ('error', models.CharField(blank=True, max_length=10000)),
                ('tx_id', models.CharField(blank=True, max_length=100)),
                ('tries', models.IntegerField(default=0)),
                ('send', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Delegates',
            fields=[
                ('username', models.CharField(max_length=20)),
                ('id', models.ForeignKey(db_column='transactionId', on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, serialize=False, to='ark.Transactions')),
            ],
            options={
                'db_table': 'delegates',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MemAccounts2Delegates',
            fields=[
                ('accountid', models.ForeignKey(db_column='accountId', on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, serialize=False, to='ark.MemAccounts')),
                ('dependentid', models.CharField(db_column='dependentId', max_length=66)),
            ],
            options={
                'db_table': 'mem_accounts2delegates',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='MemAccounts2Multisignatures',
            fields=[
                ('accountid', models.ForeignKey(db_column='accountId', on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, serialize=False, to='ark.MemAccounts')),
                ('dependentid', models.CharField(db_column='dependentId', max_length=66)),
            ],
            options={
                'db_table': 'mem_accounts2multisignatures',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Multisignatures',
            fields=[
                ('min', models.IntegerField()),
                ('lifetime', models.IntegerField()),
                ('keysgroup', models.TextField()),
                ('transactionid', models.ForeignKey(db_column='transactionId', on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, serialize=False, to='ark.Transactions')),
            ],
            options={
                'db_table': 'multisignatures',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Signatures',
            fields=[
                ('transactionid', models.ForeignKey(db_column='transactionId', on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, related_name='transactions', serialize=False, to='ark.Transactions')),
                ('publickey', models.BinaryField(db_column='publicKey')),
            ],
            options={
                'db_table': 'signatures',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Votes',
            fields=[
                ('votes', models.TextField(blank=True, null=True)),
                ('transactionid', models.ForeignKey(db_column='transactionId', on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, serialize=False, to='ark.Transactions')),
            ],
            options={
                'db_table': 'votes',
                'managed': False,
            },
        ),
    ]



# Generated by Django 2.0 on 2018-01-23 17:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0002_auto_20180123_1740'),
    ]

    operations = [
        migrations.AddField(
            model_name='transaction',
            name='res',
            field=models.CharField(max_length=10000, null=True),
        ),
    ]



from django.db import models
from django.contrib.postgres.fields import JSONField
from datetime import datetime

class Blocks(models.Model):
    id = models.CharField(primary_key=True, max_length=64)
    rowid = models.AutoField(db_column='rowId', primary_key=True)  # Field name made lowercase.
    version = models.IntegerField()
    timestamp = models.IntegerField()
    height = models.IntegerField(unique=True)
    previousblock = models.ForeignKey('self', models.DO_NOTHING, db_column='previousBlock', unique=True, blank=True, null=True)  # Field name made lowercase.
    numberoftransactions = models.IntegerField(db_column='numberOfTransactions')  # Field name made lowercase.
    totalamount = models.BigIntegerField(db_column='totalAmount')  # Field name made lowercase.
    totalfee = models.BigIntegerField(db_column='totalFee')  # Field name made lowercase.
    reward = models.BigIntegerField()
    payloadlength = models.IntegerField(db_column='payloadLength')  # Field name made lowercase.
    payloadhash = models.BinaryField(db_column='payloadHash')  # Field name made lowercase.
    generatorpublickey = models.BinaryField(db_column='generatorPublicKey')  # Field name made lowercase.
    blocksignature = models.BinaryField(db_column='blockSignature')  # Field name made lowercase.
    rawtxs = models.TextField()

    class Meta:
        managed = False
        db_table = 'blocks'


class Delegates(models.Model):
    username = models.CharField(max_length=20)
    id = models.ForeignKey('Transactions', models.DO_NOTHING, db_column='transactionId', primary_key=True)  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'delegates'


class ForksStat(models.Model):
    delegatepublickey = models.BinaryField(db_column='delegatePublicKey')  # Field name made lowercase.
    blocktimestamp = models.IntegerField(db_column='blockTimestamp')  # Field name made lowercase.
    blockid = models.CharField(db_column='blockId', max_length=64)  # Field name made lowercase.
    blockheight = models.IntegerField(db_column='blockHeight')  # Field name made lowercase.
    previousblock = models.CharField(db_column='previousBlock', max_length=64)  # Field name made lowercase.
    cause = models.IntegerField()

    class Meta:
        managed = False
        db_table = 'forks_stat'


class MemAccounts(models.Model):
    username = models.CharField(max_length=20, blank=True, null=True)
    isdelegate = models.SmallIntegerField(db_column='isDelegate', blank=True, null=True)  # Field name made lowercase.
    u_isdelegate = models.SmallIntegerField(db_column='u_isDelegate', blank=True, null=True)  # Field name made lowercase.
    secondsignature = models.SmallIntegerField(db_column='secondSignature', blank=True, null=True)  # Field name made lowercase.
    u_secondsignature = models.SmallIntegerField(db_column='u_secondSignature', blank=True, null=True)  # Field name made lowercase.
    u_username = models.CharField(max_length=20, blank=True, null=True)
    address = models.CharField(primary_key=True, max_length=36)
    publickey = models.BinaryField(db_column='publicKey', blank=True, null=True)  # Field name made lowercase.
    secondpublickey = models.BinaryField(db_column='secondPublicKey', blank=True, null=True)  # Field name made lowercase.
    balance = models.BigIntegerField(blank=True, null=True)
    u_balance = models.BigIntegerField(blank=True, null=True)
    vote = models.BigIntegerField(blank=True, null=True)
    rate = models.BigIntegerField(blank=True, null=True)
    delegates = models.TextField(blank=True, null=True)
    u_delegates = models.TextField(blank=True, null=True)
    multisignatures = models.TextField(blank=True, null=True)
    u_multisignatures = models.TextField(blank=True, null=True)
    multimin = models.SmallIntegerField(blank=True, null=True)
    u_multimin = models.SmallIntegerField(blank=True, null=True)
    multilifetime = models.SmallIntegerField(blank=True, null=True)
    u_multilifetime = models.SmallIntegerField(blank=True, null=True)
    blockid = models.CharField(db_column='blockId', max_length=64, blank=True, null=True)  # Field name made lowercase.
    nameexist = models.SmallIntegerField(blank=True, null=True)
    u_nameexist = models.SmallIntegerField(blank=True, null=True)
    producedblocks = models.IntegerField(blank=True, null=True)
    missedblocks = models.IntegerField(blank=True, null=True)
    fees = models.BigIntegerField(blank=True, null=True)
    rewards = models.BigIntegerField(blank=True, null=True)
    virgin = models.SmallIntegerField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'mem_accounts'


class MemAccounts2Delegates(models.Model):
    accountid = models.ForeignKey(MemAccounts, models.DO_NOTHING, db_column='accountId', primary_key=True)  # Field name made lowercase.
    dependentid = models.CharField(db_column='dependentId', max_length=66)  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'mem_accounts2delegates'


class MemAccounts2Multisignatures(models.Model):
    accountid = models.ForeignKey(MemAccounts, models.DO_NOTHING, db_column='accountId', primary_key=True)  # Field name made lowercase.
    dependentid = models.CharField(db_column='dependentId', max_length=66)  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'mem_accounts2multisignatures'


class MemAccounts2UDelegates(models.Model):
    accountid = models.ForeignKey(MemAccounts, models.DO_NOTHING, db_column='accountId')  # Field name made lowercase.
    dependentid = models.CharField(db_column='dependentId', max_length=66)  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'mem_accounts2u_delegates'


class MemAccounts2UMultisignatures(models.Model):
    accountid = models.ForeignKey(MemAccounts, models.DO_NOTHING, db_column='accountId')  # Field name made lowercase.
    dependentid = models.CharField(db_column='dependentId', max_length=66)  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'mem_accounts2u_multisignatures'


class MemDelegates(models.Model):
    publickey = models.CharField(db_column='publicKey', max_length=66, primary_key=True)  # Field name made lowercase.
    vote = models.BigIntegerField()
    round = models.BigIntegerField()
    producedblocks = models.IntegerField(blank=True, null=True)
    missedblocks = models.IntegerField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'mem_delegates'


class Migrations(models.Model):
    id = models.CharField(primary_key=True, max_length=22)
    name = models.TextField()

    class Meta:
        managed = False
        db_table = 'migrations'


class Multisignatures(models.Model):
    min = models.IntegerField()
    lifetime = models.IntegerField()
    keysgroup = models.TextField()
    transactionid = models.ForeignKey('Transactions', models.DO_NOTHING, db_column='transactionId', primary_key=True)  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'multisignatures'


class Peers(models.Model):
    ip = models.GenericIPAddressField(primary_key=True)
    port = models.SmallIntegerField()
    state = models.SmallIntegerField()
    os = models.CharField(max_length=64, blank=True, null=True)
    version = models.CharField(max_length=11, blank=True, null=True)
    clock = models.BigIntegerField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'peers'
        unique_together = (('ip', 'port'),)


class Signatures(models.Model):
    transactionid = models.ForeignKey('Transactions', models.DO_NOTHING, db_column='transactionId', primary_key=True,
                                      related_name='transactions')  # Field name made lowercase.
    publickey = models.BinaryField(db_column='publicKey')  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'signatures'


class Transactions(models.Model):
    id = models.CharField(primary_key=True, max_length=64)
    rowid = models.AutoField(db_column='rowId', primary_key=True)  # Field name made lowercase.
    blockid = models.ForeignKey(Blocks, models.DO_NOTHING, db_column='blockId')  # Field name made lowercase.
    type = models.SmallIntegerField()
    timestamp = models.IntegerField()
    senderpublickey = models.BinaryField(db_column='senderPublicKey')  # Field name made lowercase.
    senderid = models.CharField(db_column='senderId', max_length=36)  # Field name made lowercase.
    recipientid = models.CharField(db_column='recipientId', max_length=36, blank=True, null=True)  # Field name made lowercase.
    amount = models.BigIntegerField()
    fee = models.BigIntegerField()
    signature = models.BinaryField()
    signsignature = models.BinaryField(db_column='signSignature', blank=True, null=True)  # Field name made lowercase.
    requesterpublickey = models.BinaryField(db_column='requesterPublicKey', blank=True, null=True)  # Field name made lowercase.
    vendorfield = models.CharField(db_column='vendorField', max_length=64, blank=True, null=True)  # Field name made lowercase.
    signatures = models.TextField(blank=True, null=True)
    rawasset = models.TextField(blank=True, null=True)

    class Meta:
        managed = False
        db_table = 'transactions'


class Votes(models.Model):
    votes = models.TextField(blank=True, null=True)
    transactionid = models.ForeignKey(Transactions, models.DO_NOTHING, db_column='transactionId', primary_key=True)  # Field name made lowercase.

    class Meta:
        managed = False
        db_table = 'votes'


# a table where we store transactions and leak them to the relay node as we continue along.
class Transaction(models.Model):
    # the specific transaction we send (by using the same transaction we ensure we don't double send.
    tx = JSONField()
    desc = models.CharField(max_length=1000, null=True)

    # check/rebroadcast tx until we reach thi amount of confirmations
    confirmations = models.IntegerField(default=0)

    success = models.BooleanField(default=False)
    error = models.CharField(null=True, max_length=10000)
    tx_id = models.CharField(null=True, max_length=100, unique=True)
    tries = models.IntegerField(default=0)
    send = models.BooleanField(default=False)
    res = models.CharField(null=True, max_length=10000)
    network = models.CharField(default='dark', max_length=50)
    broadcaster = models.IntegerField(default=0)
    created = models.DateTimeField(default=datetime.now)

    class Meta:
        ordering = ['-created', 'confirmations']


class Lock(models.Model):
    name = models.CharField(blank=True, max_length=100, unique=True)
    locked = models.BooleanField(default=False)

    def __str__(self):
        return self.name



# Generated by Django 2.0 on 2018-01-24 10:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0004_auto_20180123_1756'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='transaction',
            name='tx',
        ),
    ]



# Generated by Django 2.0 on 2018-01-24 13:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0006_transaction_tx'),
    ]

    operations = [
        migrations.AddField(
            model_name='transaction',
            name='broadcaster',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='transaction',
            name='network',
            field=models.CharField(default='dark', max_length=50),
        ),
    ]



# Generated by Django 2.0 on 2018-01-23 17:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0003_transaction_res'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='tx_id',
            field=models.CharField(max_length=100, null=True, unique=True),
        ),
    ]



from django.test import TestCase
from ark.models import Lock
from ark.locks import DbLock, LockOffError, LockOnError
from ark.transactions import TX, TxBroadcaster

class LockTestCase(TestCase):

    def setUp(self):
        lock = Lock.objects.create(name="A")
        lock.save()

    def test_pre_created_lock(self):
        """Test locking behaviour of precreated lock"""
        lock = DbLock(name="A")

        lock.release(hard=True)

        # test if concurrent process is stopped
        lock.set()

        self.assertRaises(LockOnError, lock.set)

        # check if incorrect releasing of locks raises errors
        lock.release()

        self.assertRaises(LockOffError, lock.release)

    def test_auto_lock_creation(self):
        lock = DbLock("B")
        lock.set()
        self.assertRaises(LockOnError, lock.set)


class TxTestCase(TestCase):
    def test_single_tx(self):
        tx = TX(
            amount=1,
            secret="talk coral spatial wall pipe wolf orient attack soft favorite ordinary buzz",
            recipient="DA1SPukujJqfVqiGfq9yFUnnEucpxevgbA",
            network="dark",
        )

        tx.send(use_open_peers=True)
        self.assertNotEqual(tx.tx.res["success"], '0,0%')


class TxBroadCasterTestCase(TestCase):
    def setUp(self):
        """
        We bake some transactions and store them in the db.
        """
        for i in range(10):
            tx = TX(
                amount=i+1,
                secret="talk coral spatial wall pipe wolf orient attack soft favorite ordinary buzz",
                recipient="DA1SPukujJqfVqiGfq9yFUnnEucpxevgbA",
                network="dark",
            )

            tx.queue()

    def test_broadcaster(self):
        caster = TxBroadcaster(uid=0, singlerun=True)
        caster.run()






# Generated by Django 2.0 on 2018-02-03 16:31

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0008_auto_20180124_1342'),
    ]

    operations = [
        migrations.CreateModel(
            name='Lock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('job_name', models.CharField(blank=True, max_length=100, unique=True)),
                ('locked', models.BooleanField(default=False)),
            ],
        ),
        migrations.AlterModelOptions(
            name='transaction',
            options={'ordering': ['-created', 'confirmations']},
        ),
        migrations.AddField(
            model_name='transaction',
            name='created',
            field=models.DateTimeField(default=datetime.datetime.now),
        ),
    ]



# Generated by Django 2.0 on 2018-02-03 16:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0009_auto_20180203_1631'),
    ]

    operations = [
        migrations.RenameField(
            model_name='lock',
            old_name='job_name',
            new_name='name',
        ),
    ]



import django_filters
import ark.models as ark_blockchain


class TransactionFilter(django_filters.FilterSet):
    # Do case-insensitive lookups on 'name'
    class Meta:
        model = ark_blockchain.Transactions
        fields = {
            'id': ['exact'],
            'rowid': ['exact'],
            'blockid': ['exact'],
            'type':  ['exact', 'lt', 'gt', 'gte', 'lte'],
            'timestamp':  ['exact', 'lt', 'gt', 'gte', 'lte'],
            'senderid': ['exact'],
            'recipientid': ['exact'],
            'amount': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'fee': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'vendorfield': ['exact'],
            'rawasset': ['exact'],
                  }




from ark.models import Lock
from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.db.utils import IntegrityError
"""
Atomic locks to ensure only a single instance of a process is running.
"""


class LockOnError(Exception):
    pass


class LockOffError(Exception):
    pass


class DbLock:
    def __init__(self, name):
        self.name = name

    @transaction.atomic
    def set(self, hard=False):
        try:
            lock = Lock.objects.select_for_update().filter(name=self.name, locked=False).update(locked=True)
            print(lock)
            if not lock:
                try:
                    Lock.objects.select_for_update().create(name=self.name, locked=True)
                except IntegrityError:
                    if hard:
                        pass

                    raise LockOnError("Lock {} was on while trying to turn it on")

        except ObjectDoesNotExist:
            Lock.objects.select_for_update().create(name=self.name, locked=False).update(locked=True)

    @transaction.atomic
    def release(self, hard=False):
        lock = Lock.objects.select_for_update().filter(name=self.name, locked=True).update(locked=False)
        if not lock and not hard:
            raise LockOffError("Lock {} was off while trying to turn it off".format(self.name))

    def state(self):
        try:
            return Lock.objects.get(name=self.name).locked
        except ObjectDoesNotExist:
            lock = Lock.objects.select_for_update().create(name=self.name, locked=False)
            return lock.locked


from graphene_django.filter import DjangoFilterConnectionField
from graphene_django.debug import DjangoDebug
from ark.types import *
from django.db.models import Q


class Query(graphene.AbstractType):
    debug = graphene.Field(DjangoDebug, name='__debug')

    all_accounts = DjangoFilterConnectionField(AccountType)
    all_blocks = DjangoFilterConnectionField(BlocksType)
    all_dependants = DjangoFilterConnectionField(DependantType)
    all_delegates = DjangoFilterConnectionField(DelegateType)
    all_forksstats = DjangoFilterConnectionField(ForksStatType)
    all_mem2multisignatures = DjangoFilterConnectionField(MemAccounts2MultisignaturesType)
    all_memdelegates = DjangoFilterConnectionField(MemDelegatesType)
    all_migrations = DjangoFilterConnectionField(MigrationsType)
    all_multisignatures = DjangoFilterConnectionField(MultisignatureType)
    all_peers = DjangoFilterConnectionField(PeersType)
    all_signatures = DjangoFilterConnectionField(SignatureType)
    all_transactions = DjangoFilterConnectionField(TransactionType, SenderOrRecipient=graphene.String())
    all_votes = DjangoFilterConnectionField(VotesType)

    vote = graphene.Field(
        type=VotesType
    )

    transaction = graphene.Field(
        type=TransactionType,
    )

    signature = graphene.Field(
        type=SignatureType
    )

    peer = graphene.Field(
        type=PeersType
    )

    multisignature = graphene.Field(
        type=MultisignatureType
    )

    migration = graphene.Field(
        type=MigrationsType,
        id=graphene.ID(),
    )

    memdelegate = graphene.Field(
        type=MemDelegatesType
    )

    mem2multisignature = graphene.Field(
        type=MemAccounts2MultisignaturesType
    )

    forksstat = graphene.Field(
        type=ForksStatType
    )

    account = graphene.Field(
        type=AccountType,
        address=graphene.String(),
        username=graphene.String(),
        isdelegate=graphene.Int(),
        secondsignature=graphene.Int(),
        dependentid=graphene.String(),
    )

    dependent = graphene.Field(
        type=DependantType,
        accountid=graphene.String(),
        dependantid=graphene.String(),
    )

    block = graphene.Field(
        type=BlocksType,
        id=graphene.ID(),
        height=graphene.Int(),
        timestamp=graphene.Int()
    )

    delegate = graphene.Field(
        type=DelegateType,
        id=graphene.ID(),
        username=graphene.String(),
    )

    def resolve_all_votes(self, info, **kwargs):
        return ark_blockchain.Transactions.objects.prefetch_related('blockid__transactions_set__votes_set')

    def resolve_all_transactions(self, info, **kwargs):

        or_term = kwargs.get('SenderOrRecipient', '')

        if or_term:
            return ark_blockchain.Transactions.objects.prefetch_related('blockid__transactions_set__delegates_set').all().\
                filter(Q(senderid=or_term) | Q(recipientid=or_term))

        return ark_blockchain.Transactions.objects.prefetch_related('blockid__transactions_set__delegates_set').all()

    def resolve_all_signatures(self, info, **kwargs):
        return ark_blockchain.Signatures.objects.select_related('transactionid__blockid')

    def resolve_all_peers(self, info, **kwargs):
        return ark_blockchain.Peers.objects.all()

    def resolve_all_multisignatures(self, info, **kwargs):
        return ark_blockchain.Multisignatures.objects.select_related('transactionid__blockid_id')

    def resolve_all_migrations(self, info, **kwargs):
        return ark_blockchain.Migrations.objects.all()

    def resolve_all_memdelegates(self, info, **kwargs):
        return ark_blockchain.MemDelegates.objects.all()

    def resolve_all_mem2multisignatures(self, info, **kwargs):
        return ark_blockchain.MemAccounts2Multisignatures.objects.select_related('accountid')

    def resolve_all_forksstats(self, info, **kwargs):
        return ark_blockchain.ForksStat.objects.all()

    def resolve_all_blocks(self, info, **kwargs):
        return ark_blockchain.Blocks.objects.all()

    def resolve_all_accounts(self, info, **kwargs):
        return ark_blockchain.MemAccounts.objects.all()

    def resolve_all_dependents(self, info, **kwargs):
        return ark_blockchain.MemAccounts2Delegates.objects.select_related('accountid')

    def resolve_all_delegates(self, info, **kwargs):
        return ark_blockchain.Delegates.objects.all()

    def resolve_vote(self, info, **kwargs):
        transactionid = kwargs.get('transactionid')

        if transactionid is not None:
            return ark_blockchain.Votes.objects.select_related('transactionid__blockid')
        return None

    def resolve_transaction(self, info, **kwargs):
        id = kwargs.get('id')

        if id is not None:
            ark_blockchain.Transactions.objects.select_related('blockid').get(id=id)
        return None

    def resolve_signature(self, info, **kwargs):
        transactionid = kwargs.get('transactionid')

        if transactionid is not None:
            return ark_blockchain.Signatures.objects.get(transactionid=transactionid)
        return None

    def resolve_peer(self, info, **kwargs):
        ip = kwargs.get('ip')

        if ip is not None:
            return ark_blockchain.Peers.objects.get(ip=ip)
        return None

    def resolve_multisignature(self, info, **kwargs):
        transactionid = kwargs.get('transactionid')

        if transactionid is not None:
            return ark_blockchain.Multisignatures.objects.get(transactionid=transactionid)

        return None

    def resolve_migration(self, info, **kwargs):
        id = kwargs.get('id')
        name = kwargs.get('name')

        if id is not None:
            return ark_blockchain.Migrations.objects.get(id=id)

        if name is not None:
            return ark_blockchain.Migrations.objects.get(name=name)

        return None

    def resolve_memdelegate(self, info, **kwargs):
        publickey = kwargs.get('publickey')

        if publickey is not None:
            return ark_blockchain.MemDelegates.objects.get(publickey=publickey)

        return None

    def resolve_mem2multisignature(self, info, **kwargs):
        accountid = kwargs.get('accountid')
        dependentid = kwargs.get('dependentid')

        if accountid is not None:
            return ark_blockchain.MemAccounts2Multisignatures.objects.get(accountid=accountid)

        if dependentid is not None:
            return ark_blockchain.MemAccounts2Multisignatures.objects.get(dependentid=dependentid)

        return None

    def resolve_forksstat(self, info, **kwargs):
        delegatepublickey = kwargs.get('delegatepublickey')
        blocktimestamp = kwargs.get('blocktimestamp')
        blockid = kwargs.get('blockid')
        blockheight = kwargs.get('blockheight')
        previousblock = kwargs.get('previousblock')

        if delegatepublickey is not None:
            return ark_blockchain.ForksStat.objects.get(delegatepublickey=delegatepublickey)

        if blocktimestamp is not None:
            return ark_blockchain.ForksStat.objects.get(blocktimestamp=blocktimestamp)

        if blockid is not None:
            return ark_blockchain.ForksStat.objects.get(blockid=blockid)

        if blockheight is not None:
            return ark_blockchain.ForksStat.objects.get(blockheight=blockheight)

        if previousblock is not None:
            return ark_blockchain.ForksStat.objects.get(previousblock=previousblock)

        return None

    def resolve_delegate(self, info, **kwargs):
        username = kwargs.get('username')

        if username is not None:
            return ark_blockchain.Delegates.objects.get(username=username)

        return None

    def resolve_block(self, info, **kwargs):
        id = kwargs.get('id')
        timestamp = kwargs.get('timestamp')
        height = kwargs.get('height')

        if id is not None:
            return ark_blockchain.Blocks.objects.get(id=id)

        if height is not None:
            return ark_blockchain.Blocks.objects.get(height=height)

        if timestamp is not None:
            return ark_blockchain.Blocks.objects.get(timestamp=timestamp)

        return None

    def resolve_dependent(self, info, **kwargs):
        accountid = kwargs.get('accountid')

        if accountid is not None:
            return ark_blockchain.MemAccounts2Delegates.objects.get(accountid=accountid)

        return None

    def resolve_account(self, info, **kwargs):
        address = kwargs.get('address')
        username = kwargs.get('username')

        if address is not None:
            return ark_blockchain.MemAccounts.objects.prefetch_related('memaccounts2delegates_set').get(address=address)

        if username is not None:
            return ark_blockchain.MemAccounts.objects.get(username=username)

        return None



from django.conf import settings


class ArkRouter:
    """
    A router to control all database operations on models in the
    ark_app application.
    """
    def db_for_read(self, model, **hints):
        """
        Attempts to read ark_app models go to ARK_DB_NAME.
        """
        if model._meta.app_label == 'ark':
            return settings.ARK_DB_NAME
        return None

    def db_for_write(self, model, **hints):
        """
        Attempts to write ark_app models go to ARK_DB_NAME.
        """
        if model._meta.app_label == 'ark':
            return settings.ARK_DB_NAME
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        Allow relations if a model in the ark is involved.
        """
        if obj1._meta.app_label == 'ark' or \
           obj2._meta.app_label == 'ark':
           return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        """
        Make sure the ark_app app only appears in the 'ARK_DB_NAME'
        database.
        """
        if app_label == 'ark':
            return db == settings.ARK_DB_NAME
        return None


import arky.rest
from ark.models import Transaction
import random
from django.db.models import ObjectDoesNotExist
import time
from ark.locks import DbLock
from datetime import datetime


class TX:
    '''
    Get transaction from DB or bake a single transaction, and add to the transaction db
    '''
    def __init__(self, **kw):

        self.network = kw.get('network', 'dark')
        try:
            if not arky.core:
                # first time this check fails with an AttributeError, thus we initialize arky.core
                pass
        except AttributeError:
            arky.rest.use(self.network)
        # try to bake the tx ourselves if sufficient kw is supplied
        # if no variables for baking a transaction are supplied, we try to retrieve it from the DB
        try:
            self.tx = Transaction.objects.get(tx_id=kw.get('tx_id'))
        except ObjectDoesNotExist:
            trs = arky.core.crypto.bakeTransaction(
                recipientId=kw.get('recipient', None),
                amount=kw.get('amount'),
                vendorfield=kw.get('smartbridge'),
                secret=kw.get('secret'),
                secondSecret=kw.get('secondsecret', None),
            )

            self.tx = Transaction.objects.create(
                tx=trs,
                tx_id=arky.core.crypto.getId(trs),
                confirmations=0,
                desc=kw.get('desc', None),
                network=self.network
            )

    def send(self, use_open_peers=True, queue=True, **kw):
        """
        send a transaction immediately. Failed transactions are picked up by the TxBroadcaster

        :param ip: specific peer IP to send tx to
        :param port: port of specific peer
        :param use_open_peers: use Arky's broadcast method
        """

        if not use_open_peers:
            ip = kw.get('ip')
            port = kw.get('port')
            peer = 'http://{}:{}'.format(ip, port)
            res = arky.rest.POST.peer.transactions(peer=peer, transactions=[self.tx.tx])

        else:
            res = arky.core.sendPayload(self.tx.tx)

        if self.tx.success != '0.0%':
            self.tx.error = None
            self.tx.success = True
        else:
            self.tx.error = res['messages']
            self.tx.success = False

        self.tx.tries += 1
        self.tx.res = res

        if queue:
            self.tx.send = True

        self.__save()
        return res

    def confirmations(self):
        res = arky.rest.GET.api.transactions.get(id=self.tx.tx_id)

        if res.get('error', None) == 'Transaction not found':
            self.tx.confirmations = 0
        elif 'ReadTimeout' in res.get('error', ''):
            self.confirmations()
        else:
            self.tx.confirmations = res.get('transaction').get('confirmations')
        self.__save()
        return self.tx.confirmations

    def check_confirmations_or_resend(self, use_open_peers=False, **kw):
        """
        check if a tx is confirmed, else resend it.

        :param use_open_peers: select random peers fro api/peers endpoint
        """
        if self.confirmations() == 0:
            self.send(use_open_peers, **kw)

    def queue(self):
        """
        queue a transaction to be picked up by the background transaction broadcaster
        """
        self.tx.send = True
        self.__save()

    def delay(self):
        """
        save a transaction, but don't let it be picked up by the broadcaster
        """
        self.tx.send = False
        self.__save()

    def __save(self):
        self.tx.save()


class TxBroadcaster:
    """
    Broadcasts transactions and rechecks if they have been sent until a specified amount of confirmations.
    """

    def __init__(self, **kw):
        self.max_retry = kw.get('max_retry', 10)
        self.min_confs = kw.get('check_until_confs', 51)
        self.use_open_peers = kw.get('use_open_peers', True)
        self.network = kw.get('network', 'dark')
        self.uid = kw.get('uid', 0)
        self.peers = kw.get('peers', None)
        self.wait = kw.get('wait_time', 8)
        self.caster = kw.get("broadcaster", 0)
        self.singlerun = kw.get("singlerun", False)
        self.expire_tx = kw.get('expire_tx', False)
        arky.rest.use(self.network)

    def run(self):
        # we lock the run to ensure we don't get multiple instances of the same broadcaster sending out transactions
        lock = DbLock(name=self.uid)
        try:
            lock.set()
            while True:
                for x in Transaction.objects.all()\
                        .filter(confirmations__lte=self.min_confs)\
                        .filter(tries__lt=self.max_retry)\
                        .filter(send=True)\
                        .filter(broadcaster=self.caster):

                    tx = TX(tx_id=x.tx_id)

                    if self.peers:
                        peer = list(random.choice([self.peers.keys()]))[0]
                        port = self.peers.get(peer)
                    else:
                        peer = None,
                        port = None

                    # if the transaction is too old, we delete it. Mainly used to make sure we don't do double payments
                    if self.expire_tx:
                        age = datetime.now() - tx.tx.created
                        if age.seconds > self.expire_tx:
                            tx.tx.delete()
                            continue

                    tx.check_confirmations_or_resend(
                        use_open_peers=self.use_open_peers,
                        ip=peer,
                        port=port,
                )

                # mainly used for testing purposes, else the process won't terminate
                if self.singlerun:
                    lock.release()
                    break

                # lets give the network some time to incorporate our transactions before rebroadcasting again,
                # while doing so we reselect our peers
                arky.rest.use(self.network)
                time.sleep(self.wait)

        except Exception:
            lock.release()
            raise



from graphene import relay
import graphene
from graphene_django import DjangoObjectType
import ark.models as ark_blockchain
from graphene_django.converter import convert_django_field
from django.db import models
from ark.converter import BigInt, Binary


@convert_django_field.register(models.BinaryField)
def convert_column_to_string(typ, column, registry=None):
    return graphene.Field(Binary)


class VotesType(DjangoObjectType):
    class Meta:
        model = ark_blockchain.Votes
        interfaces = (relay.Node,)
        filter_fields = {
            'votes': ['exact'],
            'transactionid': ['exact']
        }


class TransactionType(DjangoObjectType):
    amount = graphene.Field(BigInt)

    class Meta:
        model = ark_blockchain.Transactions
        interfaces = (relay.Node,)
        filter_fields = {
            'id': ['exact'],
            'rowid': ['exact'],
            'blockid': ['exact'],
            'type':  ['exact', 'lt', 'gt', 'gte', 'lte'],
            'timestamp':  ['exact', 'lt', 'gt', 'gte', 'lte'],
            # 'senderpublickey': ['exact'],
            'senderid': ['exact'],
            'recipientid': ['exact'],
            'amount': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'fee': ['exact', 'lt', 'gt', 'gte', 'lte'],
            # 'signature': ['exact'],
            # 'signsignature': ['exact'],
            # 'requesterpublickey': ['exact'],
            'vendorfield': ['exact'],
            # 'signatures': ['exact'],
            'rawasset': ['exact'],
        }


class SignatureType(DjangoObjectType):
    class Meta:
        model = ark_blockchain.Signatures
        interfaces = (relay.Node,)
        filter_fields = {
            'transactionid': ['exact'],
            # 'publickey': ['exact'],
        }


class PeersType(DjangoObjectType):
    class Meta:
        model = ark_blockchain.Peers
        interfaces = (relay.Node,)
        filter_fields = {
            'ip': ['exact'],
            'port': ['exact'],
            'os': ['exact'],
            'version': ['exact'],
            'clock': ['exact', 'lt', 'gt', 'gte', 'lte'],
        }


class MultisignatureType(DjangoObjectType):
    # transactionid = graphene.ID
    class Meta:
        model = ark_blockchain.Multisignatures
        interfaces = (relay.Node,)
        filter_fields = {
            'min': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'lifetime': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'keysgroup': ['exact'],
            'transactionid': ['exact'],
        }


class MigrationsType(DjangoObjectType):
    class Meta:
        model = ark_blockchain.Migrations
        interfaces = (relay.Node,)
        filter_fields = {
            'id': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'name': ['exact']
        }


class MemDelegatesType(DjangoObjectType):
    publickey = graphene.String()

    class Meta:
        model = ark_blockchain.MemDelegates
        interfaces = (relay.Node,)
        filter_fields = {
            'publickey': ['exact'],
            'vote': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'round': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'producedblocks': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'missedblocks': ['exact', 'lt', 'gt', 'gte', 'lte'],
        }


class DelegateType(DjangoObjectType):
    username = graphene.String()
    id = graphene.ID

    class Meta:
        model = ark_blockchain.Delegates
        interfaces = (relay.Node,)
        filter_fields = {
            'username': ['exact']
        }


class ForksStatType(DjangoObjectType):
    class Meta:
        model = ark_blockchain.ForksStat
        interfaces = (relay.Node,)
        filter_fields = {
            # 'delegatepublickey': ['exact'],
            'blocktimestamp': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'blockid': ['exact'],
            'blockheight': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'previousblock': ['exact'],
            'cause': ['exact', 'lt', 'gt', 'gte', 'lte'],
        }


class MemAccounts2MultisignaturesType(DjangoObjectType):
    class Meta:
        model = ark_blockchain.MemAccounts2Multisignatures
        interfaces = (relay.Node,)
        filter_fields = {
            'accountid': ['exact'],
            'dependentid': ['exact'],

        }


class AccountType(DjangoObjectType):
    balance = graphene.Field(BigInt)
    vote = graphene.Field(BigInt)
    rewards = graphene.Field(BigInt)
    u_balance = graphene.Field(BigInt)

    class Meta:
        model = ark_blockchain.MemAccounts

        filter_fields = {
            'balance': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'vote': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'secondsignature': ['exact'],
            'isdelegate': ['exact'],
        }

        interfaces = (relay.Node,)


class DependantType(DjangoObjectType):

    class Meta:
        model = ark_blockchain.MemAccounts2Delegates
        filter_fields = {
            'accountid': ['exact'],
            'dependentid': ['exact']
        }
        interfaces = (relay.Node,)


class BlocksType(DjangoObjectType):
    rowid = graphene.Field(BigInt)
    timestamp = graphene.Field(BigInt)
    height = graphene.Field(BigInt)
    totalamount = graphene.Field(BigInt)
    totalfee = graphene.Field(BigInt)
    reward = graphene.Field(BigInt)

    class Meta:
        model = ark_blockchain.Blocks
        interfaces = (relay.Node,)

        filter_fields = {
            'height': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'timestamp': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'totalamount': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'totalfee': ['exact', 'lt', 'gt', 'gte', 'lte'],
            'numberoftransactions': ['exact', 'lt', 'gt', 'gte', 'lte'],
        }


# Generated by Django 2.0 on 2018-01-24 10:50

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0005_remove_transaction_tx'),
    ]

    operations = [
        migrations.AddField(
            model_name='transaction',
            name='tx',
            field=django.contrib.postgres.fields.jsonb.JSONField(default={'default': 1}),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.0 on 2018-01-24 13:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ark', '0007_auto_20180124_1311'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='desc',
            field=models.CharField(max_length=1000, null=True),
        ),
    ]



from setuptools import setup

setup(
    name='django-ark',
    version='0.0.2',
    packages=['ark', 'ark.migrations',],
    url='https://github.com/BlockHub/django-ark',
    license='MIT',
    author='Charles',
    author_email='karel@blockhub.nl',
    description='ark blockchain integration with Django Edit',
    install_requires=[
        'Arky','base58', 'certifi', 'cffi', 'chardet', 'Django', 'django-background-tasks',
        'django-compat', 'django-filter', 'docopt', 'ecdsa', 'ECPy', 'future', 'graphene',
        'graphene-django', 'graphql-core', 'graphql-relay', 'hidapi', 'idna', 'iso8601',
        'jsonfield', 'ledgerblue', 'Pillow', 'pkginfo', 'promise', 'protobuf', 'psycopg2',
        'pycparser', 'pycrypto', 'PyNaCl', 'python-dateutil', 'python-decouple', 'pytz',
        'requests', 'requests-toolbelt', 'Rx', 'singledispatch', 'six', 'tqdm', 'twine',
        'typing', 'urllib3'
      ],
)



from graphene.types import Scalar
from graphql.language import ast
from graphene.types.scalars import MIN_INT, MAX_INT
import binascii


class BigInt(Scalar):
    """
    BigInt is an extension of the regular Int field
        that supports Integers bigger than a signed
        32-bit integer.
    """
    @staticmethod
    def big_to_float(value):
        num = int(value)
        if num > MAX_INT or num < MIN_INT:
            return float(int(num))
        return num

    serialize = big_to_float
    parse_value = big_to_float

    @staticmethod
    def parse_literal(node):
        if isinstance(node, ast.IntValue):
            num = int(node.value)
            if num > MAX_INT or num < MIN_INT:
                return float(int(num))
            return num


class Binary(Scalar):
    """
    BinaryArray is used to convert a BinaryField to the string form
    """
    @staticmethod
    def binary_to_string(value):
        return binascii.hexlify(value).decode("utf-8")

    serialize = binary_to_string
    parse_value = binary_to_string

    @staticmethod
    def parse_literal(node):
        if isinstance(node, ast.StringValue):
            return binascii.hexlify(node.value).decode("utf-8")
